# -*- coding: utf-8 -*-
"""
# ---------------------------------------------------------------------------------------------------------
# ProjectName:  JLX-helper
# FileName:     order_service.py
# Description:  TODO
# Author:       GIGABYTE
# CreateDate:   2024/06/07
# Copyright ©2011-2024. Hunan xxxxxxx Company limited. All rights reserved.
# ---------------------------------------------------------------------------------------------------------
"""
from jlx_helper.api import JLXApi
from jlx_helper.libs import logger
from jlx_helper.config import url_map
from jlx_helper.utils import timestamp_to_datetime_str


class OrderService(object):

    def __init__(self, user: str = None, password: str = None):
        self.__user = user or url_map.get("auth_username")
        self.__password = password or url_map.get("auth_password")
        self.__client = JLXApi(domain=url_map.get("domain"), protocol=url_map.get("procotol"))

    @property
    def user(self):
        return self.__user

    def get_token(self) -> str:
        response_data = self.__client.get_authorization_token(
            method="post", path=url_map.get("get_authorization_token"), user=self.__user, password=self.__password
        )
        token = ""
        if response_data.get("code") == 0 and response_data.get("success") is True:
            token = response_data.get("data").get("token")
            expire_time = response_data.get("data").get("expireTime")
            expire_time_str = timestamp_to_datetime_str(timestamp=expire_time)
            logger.debug("获取就旅行api鉴权token值：{}，有效期至：{}".format(token, expire_time_str))
        else:
            logger.warning("获取就旅行api鉴权token失败，接口返回内容：{}".format(response_data))
        return token

    def get_flight_quotation(self, ticket_channel: str, channel_user: str, channel_password: str, departure_date: str,
                             departure_city_code: str, arrive_city_code: str, flight_no: str, cabin: str = None,
                             channel_token: str = None, jlx_token: str = None, product_type: str = 'cpa',
                             trip_type: str = 'single_trip') -> list:
        if not jlx_token:
            jlx_token = self.get_token()
            if not jlx_token:
                return list()
        response_data = self.__client.get_query_quotation(
            method="post", path=url_map.get("get_query_quotation"), token=jlx_token, ticket_channel=ticket_channel,
            channel_user=channel_user, channel_password=channel_password, departure_city_code=departure_city_code,
            arrive_city_code=arrive_city_code, departure_date=departure_date, flight_no=flight_no, cabin=cabin,
            channel_token=channel_token, product_type=product_type, trip_type=trip_type
        )
        if response_data.get("code") == 0 and response_data.get("success") is True:
            return response_data.get("data") if isinstance(response_data.get("data"), list) else list()
        else:
            logger.warning("获取就旅行航班报价信息失败，接口返回内容：{}".format(response_data))
        return list()

    def gen_jlx_order(self, ticket_channel: str, channel_user: str, order_id: str, departure_date: str,
                      departure_city_code: str, arrive_city_code: str, flight_no: str, passengers: list,
                      internal_phone: str, cabin: str = None, jlx_token: str = None, price_context: str = None,
                      conditions: list = None, trip_type: str = 'single_trip') -> dict:
        if not jlx_token:
            jlx_token = self.get_token()
            if not jlx_token:
                return dict()
        response_data = self.__client.gen_service_order(
            method="post", path=url_map.get("gen_service_order"), token=jlx_token, ticket_channel=ticket_channel,
            channel_user=channel_user, order_id=order_id, departure_city_code=departure_city_code,
            arrive_city_code=arrive_city_code, departure_date=departure_date, flight_no=flight_no,
            passengers=passengers, internal_phone=internal_phone, price_context=price_context, conditions=conditions,
            cabin=cabin, trip_type=trip_type
        )
        if response_data.get("code") == 0 and response_data.get("success") is True:
            return dict(
                order_id=response_data.get("data").get("orderNo"), price=response_data.get("data").get("price"),
                order_context=response_data.get("data").get("orderContext"),
                raw_order_id=response_data.get("data").get("yourOrderNo"),
            )
        else:
            logger.warning("创建就旅行出票订单失败，接口返回内容：{}".format(response_data))
        return dict()

    def payment_jlx_order(self, ticket_channel: str, channel_user: str, jlx_order_id: str, pay_type: str,
                          jlx_token: str = None, bank_card_info: dict = None, pay_user_info: dict = None) -> dict:
        if not jlx_token:
            jlx_token = self.get_token()
            if not jlx_token:
                return dict()
        response_data = self.__client.payment_service_order(
            method="post", path=url_map.get("payment_service_order"), token=jlx_token, ticket_channel=ticket_channel,
            channel_user=channel_user, jlx_order_id=jlx_order_id, pay_type=pay_type, bank_card_info=bank_card_info,
            pay_user_info=pay_user_info
        )
        if response_data.get("code") == 0 and response_data.get("success") is True:
            # 0出票中，1出票完成，2出票失败
            return dict(
                jlx_order_id=response_data.get("data").get("orderNo"), price=response_data.get("data").get("price"),
                order_context=response_data.get("data").get("orderContext"),
                status=response_data.get("data").get("status"), bill_id=response_data.get("data").get("billNo"),
            )
        else:
            logger.warning("支付就旅行订单<{}>失败，接口返回内容：{}".format(jlx_order_id, response_data))
        return dict()

    def get_jlx_order_itinerary_info(self, ticket_channel: str, channel_user: str, jlx_order_id: str,
                                     jlx_token: str = None, order_context: str = None) -> dict:
        if not jlx_token:
            jlx_token = self.get_token()
            if not jlx_token:
                return dict()
        response_data = self.__client.get_itinerary_info(
            method="post", path=url_map.get("payment_service_order"), token=jlx_token, ticket_channel=ticket_channel,
            channel_user=channel_user, jlx_order_id=jlx_order_id, order_context=order_context
        )
        if response_data.get("code") == 0 and response_data.get("success") is True:
            # 0出票中，1出票完成，2出票失败
            return dict(
                jlx_order_id=response_data.get("data").get("orderNo"),
                error_msg=response_data.get("data").get("errorMsg"), status=response_data.get("data").get("status"),
                tickets=response_data.get("data").get("tickets") if isinstance(
                    response_data.get("data"), list) else list(),
            )
        else:
            logger.warning("获取就旅行订单<{}>的票号失败，接口返回内容：{}".format(jlx_order_id, response_data))
        return dict()


if __name__ == "__main__":
    ser = OrderService()
    t = ser.get_token()
    logger.info(t)



# -*- coding: utf-8 -*-
"""
# ---------------------------------------------------------------------------------------------------------
# ProjectName:  JLX-helper
# FileName:     http_client.py
# Description:  TODO
# Author:       GIGABYTE
# CreateDate:   2024/06/07
# Copyright ©2011-2024. Hunan xxxxxxx Company limited. All rights reserved.
# ---------------------------------------------------------------------------------------------------------
"""
import requests
from json import dumps
from copy import deepcopy
from jlx_helper.libs import logger
from jlx_helper.utils import covert_dict_key_to_lower, get_html_title


class HttpService(object):
    __url = None
    __domain = None
    __time_out = 120
    __protocol = None
    __headers: dict = {
        "Content-Type": "application/json; charset=UTF-8",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) " +
                      "Chrome/123.0.0.0 Safari/537.36"
    }

    def __init__(self, domain: str, protocol: str) -> None:
        self.__domain = domain
        self.__protocol = protocol

    def send_request(self, method: str, path: str, params: dict = None, data: dict = None, json: dict = None,
                     headers: dict = None) -> dict:
        if headers and isinstance(headers, dict):
            __headers = deepcopy(self.__headers)
        else:
            __headers = self.__headers
        self.__url = "{}://{}{}".format(self.__protocol, self.__domain, path)
        # 发送HTTP请求
        logger.debug(
            "发起http请求，url: {}, 方法：{}，请求params参数：{}，请求headers参数：{}，请求data参数：{}，请求json参数：{}".format(
                self.__url,
                method,
                dumps(params) if params else "{}",
                dumps(__headers),
                dumps(data) if data else "{}",
                dumps(json) if json else "{}"
            )
        )
        return self.__send_http_request(
            method=method, params=params, data=data, json=json, url=self.__url, time_out=self.__time_out,
            headers=__headers
        )

    @classmethod
    def __send_http_request(cls, url: str, method: str, time_out: int, headers: dict = None, params: dict = None,
                            data: dict = None, json: dict = None) -> dict:
        # 实际发送HTTP请求的内部方法
        # 使用 requests 库发送请求
        method = method.lower().strip()
        if method in ("get", "post"):
            try:
                if method == "get":
                    response = requests.get(url, params=params, verify=False, timeout=time_out, headers=headers)
                else:
                    response = requests.post(
                        url, params=params, json=json, data=data, verify=False, timeout=time_out, headers=headers
                    )
                result = cls.__parse_data_response(url=url, response=response)
            except Exception as e:
                logger.error("调用url<{}>异常，原因：{}".format(url, str(e)))
                result = dict(code=500, message=str(e), data=dict())
        else:
            result = dict(code=400, message="Unsupported HTTP method: {}".format(method), data=dict())
        return result

    @classmethod
    def __parse_data_response(cls, url: str, response: requests.Response) -> dict:
        # 获取 Content-Type 头信息
        content_type = response.headers.get('Content-Type')
        # 判断返回的内容类型
        if 'application/json' in content_type or 'text/json' in content_type:
            # JSON 类型
            data = covert_dict_key_to_lower(d=response.json())
        elif 'text/plain' in content_type:
            # 纯文本类型
            data = dict(code=response.status_code, message=get_html_title(
                html=response.text), data=response.text)
        else:
            # 其他类型，默认视为二进制内容
            content = response.content.decode('utf-8')
            data = dict(code=response.status_code,
                        message=get_html_title(html=content), data=content)
        logger.debug("调用url: {} 的正常返回值为：{}".format(url, dumps(data)))
        return data



# -*- coding: utf-8 -*-
"""
# ---------------------------------------------------------------------------------------------------------
# ProjectName:  JLX-helper
# FileName:     utils.py
# Description:  TODO
# Author:       GIGABYTE
# CreateDate:   2024/06/07
# Copyright ©2011-2024. Hunan xxxxxxx Company limited. All rights reserved.
# ---------------------------------------------------------------------------------------------------------
"""
import re
from datetime import datetime

dt_standar_format = '%Y-%m-%d %H:%M:%S'


def covert_dict_key_to_lower(d: dict) -> dict:
    result = dict()
    for key, value in d.items():
        if isinstance(key, str):
            key_new = key.lower()
            result[key_new] = value
    return result


def get_html_title(html: str) -> str:
    # 使用正则表达式提取目标字符串
    pattern = '<title>(.*?)</title>'
    match = re.search(pattern, html)
    if match:
        title = match.group(1)
    else:
        title = "Abnormal HTML document structure"
    return title


def timestamp_to_datetime(timestamp: int) -> datetime:
    if len(str(timestamp)) == 13:
        # 将 13 位时间戳转换为秒
        timestamp = timestamp / 1000.0

    # 将时间戳转换为 datetime 对象
    dt_object = datetime.fromtimestamp(timestamp)
    return dt_object


def timestamp_to_datetime_str(timestamp: int) -> str:
    dt_object = timestamp_to_datetime(timestamp=timestamp)
    return dt_object.strftime(dt_standar_format)
